<?php

// PHP function which generates a full HTML page to browse Conley-Morse
// graphs and phase space portraits.
//
// Copyright (C) 2006-2008 by Pawel Pilarczyk, http://www.pawelpilarczyk.com/
//
// This file is part of my research software package.  This is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

// Started in 2006. Last revision: May 18, 2008.


require ("template.php");
require ("imgclick.php");


// --------------------------------------------------
// -------------- auxiliary functions ---------------
// --------------------------------------------------

// Computes the max distance between the coordinates.
function maxdist ($p, $dim)
{
	$mincoord = $p [0];
	$maxcoord = $p [0];
	for ($i = 1; $i < $dim; ++ $i)
	{
		if ($mincoord > $p [$i])
			$mincoord = $p [$i];
		if ($maxcoord < $p [$i])
			$maxcoord = $p [$i];
	}
	return ($maxcoord - $mincoord);
} /* maxdist */

// Retrieves the coordinates of the point
// or sets up some initial ones.
function getcoordinates ($dim, $max)
{
	// decode coordinates from the input form and adjust them to min/max
	$q = img_coord (true);
	$p = array ();
	for ($i = 0; $i < $dim; ++ $i)
	{
		if ($i < count ($q))
			$p [$i] = $q [$i];
		else
			$p [$i] = $_REQUEST ['p' . strval ($i)];
		if (empty ($p [$i]) && ($p [$i] !== '0') && ($p [$i] !== 0))
			$p [$i] = intval ($max [$i] / 4);
		$p [$i] = intval ($p [$i]);
		if ($p [$i] < 0)
			$p [$i] = 0;
		if ($p [$i] >= $max [$i])
			$p [$i] = $max [$i] - 1;
	}
	return $p;
} /* getcoordinates */

// Adjusts the coordinates to the diagonal if necessary.
// Uses old coordinates if available.
// Returns the adjusted coordinates.
function adjdiag ($p, $dim, $diagonal)
{
	if ($diagonal <= 0)
		return $p;
	if (maxdist ($p, $dim) < $diagonal)
		return $p;

	// if only one coordinate changed, keep its value
	$index = -1;
	for ($i = 0; $i < $dim; ++ $i)
	{
		$old [$i] = $_REQUEST ['old' . strval ($i)];
		if ((empty ($old [$i]) && ($old [$i] !== '0')) ||
			(intval ($old [$i]) == $p [$i]))
			continue;
		if ($index >= 0)
		{
			$index = -1;
			break;
		}
	//	if (abs ($p [$i] - intval ($old [$i])) != 1)
	//		break;
		$index = $i;
	}
	if ($index >= 0)
		$p [$i] = intval ($old [$i]);

	// adjust coordinates so that the point is close to the diagonal
	if ($index < 0)
		$index = 0;
	$mincoord = $p [$index];
	$maxcoord = $p [$index];
	for ($i = 0; $i < $dim; ++ $i)
	{
		if ($i == $index)
			continue;
		if ($p [$i] - $mincoord >= $diagonal)
			$p [$i] = $mincoord + $diagonal - 1;
		if ($maxcoord - $p [$i] >= $diagonal)
			$p [$i] = $maxcoord - $diagonal + 1;
		if ($mincoord > $p [$i])
			$mincoord = $p [$i];
		if ($maxcoord < $p [$i])
			$maxcoord = $p [$i];
	}
	return $p;
} /* adjdiag */

// Creates a field with parameters to set.
function paramfield ($number, $value)
{
	$field = <<< END_FIELD
Param${number}: <input type="text" name="p${number}" value="${value}" size="5"/>
<input type="submit" value="+"
onclick="document.forms.xy.p${number}.value=parseInt(document.forms.xy.p${number}.value)+1;return true;"/>
<input type="submit" value="-"
onclick="document.forms.xy.p${number}.value=parseInt(document.forms.xy.p${number}.value)-1;return true;"/>
<input type="hidden" name="old${number}" value="${value}"/>
&nbsp; &nbsp; &nbsp;
END_FIELD;
	return $field;
} /* paramfield */


// --------------------------------------------------
// -------------------- BROWSER ---------------------
// --------------------------------------------------

// Creates a web page for the browser of Morse decomposition.
// Arguments:
// $dim - dimension of the parameter space (e.g., 2)
// $maxx - the size of the parameter area in the horiz. direction (e.g., 64)
// $maxy - the size of the parameter area in the vert. direction (e.g., 64)
// $diagonal - diagonal option: 0 - none, 1 - diag only, >1 - more
// $prefixes - array of prefixes of the images
// $suffixes - array of suffixes of the images;
//   the core part of images consists of the coordinates separated with "_",
//   e.g., "zipped/gifunzip.php?zip=cont2.zip&gif=p_" + "1_12_3" + "c.gif"
// $descriptions - array of descriptions of images
// $append - text to append to the displayed image only
// Returns the complete text of a HTML page for the browser.
function browser ($dim, $max, $diagonal,
	$prefixes, $suffixes, $descriptions, $maxsize, $message = '')
{
	$p = getcoordinates ($dim, $max);
	$p = adjdiag ($p, $dim, $diagonal);

	// change max size if requested to
	$maxsizereq = $_REQUEST ['max'];
	if (!empty ($maxsizereq))
		$maxsize = $maxsizereq;

	// create the picture links and checkboxes which pictures to show
	$images = '';
	$showboxes = "<input type=\"hidden\" name=\"visiblefields\" " .
		"value=\"present\" />\n";
	$visiblefields = !empty ($_REQUEST ['visiblefields']);
	for ($n = 0; $n < count ($prefixes); ++ $n)
	{
		$checked = '';
		if (!$visiblefields ||
			!empty ($_REQUEST ['visible' . strval ($n)]))
		{
			$checked = " checked=\"checked\"";
		}
		$showboxes .= "<input type=\"checkbox\" " .
			"name=\"visible${n}\" value=\"1\"${checked}/>\n";
		if (empty ($checked))
			continue;
		$imgname = $prefixes [$n];
		for ($i = 0; $i < $dim; ++ $i)
		{
			$imgname .= ($i ? "_" : "") . strval ($p [$i]);
		}
		$imgname .= $suffixes [$n];
		$desc = $descriptions [$n];
		$images .= " <a href=\"${imgname}\"><img border=\"1\"";
		if (!empty ($desc))
			$images .= " name=\"${desc}\" alt=\"${desc}\"";
		$append = '';
		if ($maxsize)
			$append = "&max=${maxsize}";
		$images .= " src=\"${imgname}${append}\"/></a>";
	}

	// create fields for coordinates
	$fields = '';
	for ($i = 0; $i < $dim; ++ $i)
	{
		$fields .= paramfield ($i, $p [$i]);
	}

	// a field with the maximal size of the images
	$maxfield = "<input type=\"text\" name=\"max\" " .
		"value=\"${maxsize}\" size=\"5\" />";

	// prepare the limit on coordinates to display
	$ranges = '';
	for ($i = 0; $i < $dim; ++ $i)
	{
		if ($i)
			$ranges .= ", ";
		$ranges .= '0-' . strval ($max [$i] - 1);
	}

	// prepare the note about the diagonal
	$diaginfo = '';
	if ($diagonal == 1)
	{
		$diaginfo = "; at the diagonal";
	}
	else if ($diagonal > 1)
	{
		$dist = $diagonal - 1;
		$diaginfo = "; at most ${dist} from the diagonal";
	}

	// prepare the note about image resacling
	$maxinfo = '';
	$maxvalues = array ();
	if (preg_match ('/\&max=(\d+)/', $append, $maxvalues))
	{
		$maxinfo = "With the current settings, " .
			"the images larger than " .
			"${maxvalues[1]}x${maxvalues[1]} " .
			"are displayed down-scaled; click each image " .
			"to see it full-size.";
	}

	$page = html_begin ("Conley-Morse Graphs Browser", "", "", false);
	
	$page .= <<< END_PAGE
<h1>Conley-Morse Graphs and Morse Decompositions</h1>

<form name="xy" action="" method="get">
<p align="center"><table border="0" cellpadding="8" cellspacing="0">
<tr><td align="center">${fields} <input type="submit" value="show"/></td></tr>
<tr><td align="center">Image selection: ${showboxes} &nbsp; &nbsp; &nbsp;
Max image size: ${maxfield}</td></tr>
</table></p>
</form>
${message}
<p align="center">${images}</p>
</div>

<hr/>

<div id="small">
<p>Quick instructions for the browser: Enter parameter box coordinates
(in the range ${ranges}${diaginfo}) and click "show",
or click the +/- buttons to modify the current parameter box coordinates.
In order to exclude any of the subsequently displayed pictures,
uncheck the corresponding tickbox. The rightmost form entry
is the picture size limit (in pixels) which can be adjusted if necessary.
${maxinfo}</p>

<p>Conley-Morse graphs: Each vertex in the graph corresponds
to a combinatorial Morse set. The subsequent number of the set
(starting at 0) is followed by a colon and the number of boxes in the set.
Then the homology groups over Z are indicated, and homomorphisms in homology
are briefly described, followed by the eigenvalues for each level.</p>

<p>Morse decompositions: Each combinatorial Morse set is indicated
in a different color. The order of colors is indicated at the color bar
at the bottom of the picture.</p>
END_PAGE;
	$page .= html_end ();
	return $page;
}

?>